#ifndef AJLIB_COLOUR_H
#define AJLIB_COLOUR_H

#include "fix.h"

typedef Fixed<8, Word> UFix8p8;

class Colour
{
public:
    Colour() { }
    Colour(const Colour& x) : _r(x._r), _g(x._g), _b(x._b) { }
    const Colour& operator=(const Colour& x) { _r=x._r; _g=x._g; _b=x._b; return *this; }
    const Colour& operator+=(const Colour& x) { _r+=x._r; _g+=x._g; _b+=x._b; return *this; }
    const Colour& operator-=(const Colour& x) { _r-=x._r; _g-=x._g; _b-=x._b; return *this; }
    const Colour& operator*=(const UFix8p8& x) { _r*=x; _g*=x; _b*=x; return *this; }
    const Colour& operator/=(const UFix8p8& x) { _r/=x; _g/=x; _b/=x; return *this; }
    const Colour& operator<<=(int n) { _r<<=n; _g<<=n; _b<<=n; return *this; }
    const Colour& operator>>=(int n) { _r>>=n; _g>>=n; _b>>=n; return *this; }
    Colour operator+(const Colour& x) const { Colour y=*this; return y+=x; }
    Colour operator-(const Colour& x) const { Colour y=*this; return y-=x; }
    Colour operator*(const UFix8p8& x) const { Colour y=*this; return y*=x; }
    Colour operator/(const UFix8p8& x) const { Colour y=*this; return y/=x; }
    Colour operator<<(int n) const { Colour y=*this; return y<<=n; }
    Colour operator>>(int n) const { Colour y=*this; return y>>=n; }

    DWord asDWord() const
    {
        return
            (static_cast<DWord>(_r.intPart())    )
          | (static_cast<DWord>(_g.intPart())<< 8)
          | (static_cast<DWord>(_b.intPart())<<16);
    }

    static Colour black() { return Colour(0, 0, 0); }
    static Colour white() { return Colour(255, 255, 255); }
    Colour(UFix8p8 r, UFix8p8 g, UFix8p8 b) : _r(r), _g(g), _b(b) { }
protected:
    UFix8p8 _r;
    UFix8p8 _g;
    UFix8p8 _b;
};

#endif // AJLIB_COLOUR_H